<?php
/**
 * Plugin Name: WooCommerce Swedbank gateway V2
 * Plugin URI: https://sppdemoshop.eu/
 * Description: Swedbank gateway V2 for bank transfer and Debit/Credit Card payment support for WooCommerce.
 * Author: Darius Augaitis
 * Author URI:
 * Version: 2.0
 * Text Domain: swedbank-plugin
 * Domain Path: /languages
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 */
if (!defined('ABSPATH')) {
    exit; // Exit if accessed directly
}

add_action('init', 'do_output_buffer');
function do_output_buffer() {
    ob_start();
}

add_action('wp_enqueue_scripts', 'callback_for_setting_up_scripts');

function callback_for_setting_up_scripts() {

    wp_register_style( 'swedbankv2', plugins_url('css/swedbank_plugin.css',__FILE__ ) );
    wp_enqueue_style( 'swedbankv2' );
}

add_action('plugins_loaded', 'init_swedbank_v2_lt_gateway_class');

function add_swedbank_v2_lt_gateway_class($methods)
{
    $methods[] = 'WC_Gateway_Swedbank_lt_v2';
    return $methods;
}



add_filter('woocommerce_payment_gateways', 'add_swedbank_v2_lt_gateway_class');

add_filter('woocommerce_available_payment_gateways', 'filter_gateways2', 1);

function filter_gateways2($gateways)
{
    global $woocommerce;

    // return $gateways
    if (isset($gateways['swedbankv2'])) {

        if ($gateways['swedbankv2']->settings['card_lt'] === 'yes') {
            if (!isset($gateways['swedbank_v2_card_lt'])) {
                $gateways['swedbank_v2_card_lt'] = clone $gateways['swedbankv2'];
                $gateways['swedbank_v2_card_lt']->icon = '/wp-content/plugins/woocommerce-gateway-swedbank-v2/image/cards.png';
                $gateways['swedbank_v2_card_lt']->id = 'swedbank_v2_card_lt';
                $gateways['swedbank_v2_card_lt']->title = __('Payment card', 'swedbank-plugin');
                $gateways['swedbank_v2_card_lt']->description = '';
            }
        }


        if ($gateways['swedbankv2']->settings['swedbank_v2_lt'] === 'yes') {
            if (!isset($gateways['swedbank_v2_swedbank_v2_lt'])) {
                $gateways['swedbank_v2_swedbank_v2_lt'] = clone $gateways['swedbankv2'];
                $gateways['swedbank_v2_swedbank_v2_lt']->icon = '/wp-content/plugins/woocommerce-gateway-swedbank-v2/image/logo.swedbank.jpg';
                $gateways['swedbank_v2_swedbank_v2_lt']->id = 'swedbank_v2_swedbank_v2_lt';
                $gateways['swedbank_v2_swedbank_v2_lt']->title = 'Swedbank';
                $gateways['swedbank_v2_swedbank_v2_lt']->description = '';
            }
        }

        if ($gateways['swedbankv2']->settings['seb_lt'] === 'yes') {
            if (!isset($gateways['swedbank_v2_seb_lt'])) {
                $gateways['swedbank_v2_seb_lt'] = clone $gateways['swedbankv2'];
                $gateways['swedbank_v2_seb_lt']->icon = '/wp-content/plugins/woocommerce-gateway-swedbank-v2/image/SEB_k_sw.png';
                $gateways['swedbank_v2_seb_lt']->id = 'swedbank_v2_seb_lt';
                $gateways['swedbank_v2_seb_lt']->title = 'SEB';
                $gateways['swedbank_v2_seb_lt']->description = '';
            }
        }

        if ($gateways['swedbankv2']->settings['dnb_lt'] === 'yes') {
            if (!isset($gateways['swedbank_v2_dnb_lt'])) {
                $gateways['swedbank_v2_dnb_lt'] = clone $gateways['swedbankv2'];
                $gateways['swedbank_v2_dnb_lt']->icon = '/wp-content/plugins/woocommerce-gateway-swedbank-v2/image/dnb_logo.png';
                $gateways['swedbank_v2_dnb_lt']->id = 'swedbank_v2_dnb_lt';
                $gateways['swedbank_v2_dnb_lt']->title = 'DNB';
                $gateways['swedbank_v2_dnb_lt']->description = '';
            }
        }

        if ($gateways['swedbankv2']->settings['card_lv'] === 'yes') {
            if (!isset($gateways['swedbank_v2_card_lv'])) {
                $gateways['swedbank_v2_card_lv'] = clone $gateways['swedbankv2'];
                $gateways['swedbank_v2_card_lv']->icon = '/wp-content/plugins/woocommerce-gateway-swedbank-v2/image/cards.png';
                $gateways['swedbank_v2_card_lv']->id = 'swedbank_v2_card_lv';
                $gateways['swedbank_v2_card_lv']->title = __('Payment card', 'swedbank-plugin');
                $gateways['swedbank_v2_card_lv']->description = '';
            }
        }


        if ($gateways['swedbankv2']->settings['swedbank_v2_lv'] === 'yes') {
            if (!isset($gateways['swedbank_v2_swedbank_v2_lv'])) {
                $gateways['swedbank_v2_swedbank_v2_lv'] = clone $gateways['swedbankv2'];
                $gateways['swedbank_v2_swedbank_v2_lv']->icon = '/wp-content/plugins/woocommerce-gateway-swedbank-v2/image/logo.swedbank.jpg';
                $gateways['swedbank_v2_swedbank_v2_lv']->id = 'swedbank_v2_swedbank_v2_lv';
                $gateways['swedbank_v2_swedbank_v2_lv']->title = 'Swedbank';
                $gateways['swedbank_v2_swedbank_v2_lv']->description = '';
            }
        }

        if ($gateways['swedbankv2']->settings['seb_lv'] === 'yes') {
            if (!isset($gateways['swedbank_v2_seb_lv'])) {
                $gateways['swedbank_v2_seb_lv'] = clone $gateways['swedbankv2'];
                $gateways['swedbank_v2_seb_lv']->icon = '/wp-content/plugins/woocommerce-gateway-swedbank-v2/image/SEB_k_sw.png';
                $gateways['swedbank_v2_seb_lv']->id = 'swedbank_v2_seb_lv';
                $gateways['swedbank_v2_seb_lv']->title = 'SEB';
                $gateways['swedbank_v2_seb_lv']->description = '';
            }
        }

        if ($gateways['swedbankv2']->settings['card_ee'] === 'yes') {
            if (!isset($gateways['swedbank_v2_card_ee'])) {
                $gateways['swedbank_v2_card_ee'] = clone $gateways['swedbankv2'];
                $gateways['swedbank_v2_card_ee']->icon = '/wp-content/plugins/woocommerce-gateway-swedbank-v2/image/cards.png';
                $gateways['swedbank_v2_card_ee']->id = 'swedbank_v2_card_ee';
                $gateways['swedbank_v2_card_ee']->title = __('Payment card', 'swedbank-plugin');
                $gateways['swedbank_v2_card_ee']->description = '';
            }
        }


        if ($gateways['swedbankv2']->settings['swedbank_v2_ee'] === 'yes') {
            if (!isset($gateways['swedbank_v2_swedbank_v2_ee'])) {
                $gateways['swedbank_v2_swedbank_v2_ee'] = clone $gateways['swedbankv2'];
                $gateways['swedbank_v2_swedbank_v2_ee']->icon = '/wp-content/plugins/woocommerce-gateway-swedbank-v2/image/logo.swedbank.jpg';
                $gateways['swedbank_v2_swedbank_v2_ee']->id = 'swedbank_v2_swedbank_v2_ee';
                $gateways['swedbank_v2_swedbank_v2_ee']->title = 'Swedbank';
                $gateways['swedbank_v2_swedbank_v2_ee']->description = '';
            }
        }

        unset($gateways['swedbankv2']);
    }

    return $gateways;
}

function init_swedbank_v2_lt_gateway_class()
{


    class WC_Gateway_Swedbank_lt_v2 extends WC_Payment_Gateway
    {

        /** @var array Array of locales */
        public $locale;

        /**
         * Constructor for the gateway.
         */
        public function __construct()
        {

            $this->id = 'swedbankv2';
            $this->icon = apply_filters('woocommerce_bacs_icon', '');
            $this->has_fields = true;
            $this->method_title = __('Swedbank V2', 'woocommerce');
            $this->method_description = __('Allows payments by Debit/Credit Card and direct bank/wire/banklink transfer using Swedbank.', 'woocommerce');

            // Load the settings.
            $this->init_form_fields();
            $this->init_settings();

            // Define user set variables
            $this->title = __('Swedbank V2', 'woocommerce');
            $this->description = $this->get_option('description');
            $this->instructions = $this->get_option('instructions');


            // Actions
            add_action('woocommerce_update_options_payment_gateways_' . $this->id, array($this, 'process_admin_options'));

            add_action('woocommerce_thankyou_swedbank_v2_lt', array($this, 'thankyou_page'));

            add_action('woocommerce_email_before_order_table', array($this, 'email_instructions'), 10, 3);
        }

        /**
         * Initialise Gateway Settings Form Fields.
         */
        public function init_form_fields()
        {

            $this->form_fields = array(

                'enabled' => array(
                    'title' => __('Enable/Disable', 'woocommerce'),
                    'type' => 'checkbox',
                    'label' => __('To use this plugin you need to accept <a href="https://ib.swedbank_v2_lt.lt/static/pdf/business/cash/cashflow/cardservice/TC_for_the_use_of_Swedbank_Payment_Portal_WooCommerce_Module_v20170421.pdf" target="_blank">terms and conditions</a>', 'woocommerce'),
                    'default' => 'no'
                ),
                'debuging' => array(
                    'title' => __('Enable/Disable', 'woocommerce'),
                    'type' => 'checkbox',
                    'label' => __('Debugging', 'woocommerce'),
                    'description' => __('Storing transaction XML to logs. This should be turned ON only if needed to track where operation fails, otherwise keep disabled. (wp-content/uploads/wc-logs/swedbankv2.log)', 'woocommerce'),
                    'default' => 'no'
                ),
                'notifik' => array(
                    'title' => __('not', 'woocommerce'),
                    'type' => 'label',
                    'label' => __('', 'woocommerce'),
                    'description' => __('Storing transaction XML to logs. This should be turned ON only if needed to track where operation fails, otherwise keep disabled. (wp-content/uploads/wc-logs/swedbankv2.log)', 'woocommerce'),
                    'default' => 'no'
                ),
                'order_status' => array(
                    'title' => __('Order status after payment', 'woocommerce'),
                    'type' => 'select',
                    'options' => wc_get_order_statuses(),
                ),
                'instructions' => array(
                    'title' => __('Other information', 'woocommerce'),
                    'type' => 'textarea',
                    'description' => __('Other information will be visible on the thank you page and emails.', 'woocommerce'),
                    'default' => '',
                    'desc_tip' => true,
                ),
                'card_lt' => array(
                    'title' => __('Enable/Disable', 'woocommerce'),
                    'type' => 'checkbox',
                    'label' => __('Card payment for Lithuania', 'woocommerce'),
                    'default' => 'no'
                ),
                'swedbank_v2_lt' => array(
                    'title' => __('Enable/Disable', 'woocommerce'),
                    'type' => 'checkbox',
                    'label' => __('Swedbank for Lithuania', 'woocommerce'),
                    'default' => 'no'
                ),
                'seb_lt' => array(
                    'title' => __('Enable/Disable', 'woocommerce'),
                    'type' => 'checkbox',
                    'label' => __('SEB for Lithuania', 'woocommerce'),
                    'default' => 'no'
                ),
                'dnb_lt' => array(
                    'title' => __('Enable/Disable', 'woocommerce'),
                    'type' => 'checkbox',
                    'label' => __('DNB for Lithuania', 'woocommerce'),
                    'default' => 'no'
                ),
                'testmode_lt' => array(
                    'title' => __('Enable/Disable', 'woocommerce'),
                    'type' => 'checkbox',
                    'label' => __('Test mode', 'woocommerce'),
                    'default' => 'no'
                ),
                /*'banklinklng_lt' => array(
                    'title' => __('Banklink language on bank for Lithuania', 'woocommerce'),
                    'type' => 'select',
                    'options' => array(
                        'LT' => 'LT',
                        'RU' => 'RU',
                        'EN' => 'EN'
                    ),
                ),*/
                'testvtid_lt' => array(
                    'title' => __('Test ID / vTID for Lithuania', 'woocommerce'),
                    'type' => 'text',
                    'description' => __('This information you got in you wellcome email from Swedbank. If you don\'t have this information please fill form and we will contact you or you can contact us by above provided contact information', 'woocommerce'),
                    'desc_tip' => true,
                    'required' => true
                ),
                'testpass_lt' => array(
                    'title' => __('Test Password for Lithuania', 'woocommerce'),
                    'type' => 'text',
                    'description' => __('This information you got in you wellcome email from Swedbank. If you don\'t have this information please fill form and we will contact you or you can contact us by above provided contact information', 'woocommerce'),
                    'desc_tip' => true,
                ),
                'vtid_lt' => array(
                    'title' => __('ID / vTID for Lithuania', 'woocommerce'),
                    'type' => 'text',
                    'default' => __('', 'woocommerce'),
                    'desc_tip' => true,
                ),
                'pass_lt' => array(
                    'title' => __('Password for Lithuania', 'woocommerce'),
                    'type' => 'text',
                    'default' => __('', 'woocommerce'),
                    'desc_tip' => true,
                ),
                'card_lv' => array(
                    'title' => __('Enable/Disable', 'woocommerce'),
                    'type' => 'checkbox',
                    'label' => __('Card payment for Latvia', 'woocommerce'),
                    'default' => 'no'
                ),
                'swedbank_v2_lv' => array(
                    'title' => __('Enable/Disable', 'woocommerce'),
                    'type' => 'checkbox',
                    'label' => __('Swedbank for Latvia', 'woocommerce'),
                    'default' => 'no'
                ),
                'seb_lv' => array(
                    'title' => __('Enable/Disable', 'woocommerce'),
                    'type' => 'checkbox',
                    'label' => __('SEB for Latvia', 'woocommerce'),
                    'default' => 'no'
                ),
                'testmode_lv' => array(
                    'title' => __('Enable/Disable', 'woocommerce'),
                    'type' => 'checkbox',
                    'label' => __('Test mode', 'woocommerce'),
                    'default' => 'no'
                ),
                /*'banklinklng_lv' => array(
                    'title' => __('Banklink language on bank for Latvia', 'woocommerce'),
                    'type' => 'select',
                    'options' => array(
                        'LV' => 'LV',
                        'RU' => 'RU',
                        'EN' => 'EN'
                    ),
                ),*/
                'testvtid_lv' => array(
                    'title' => __('Test ID / vTID for Latvia', 'woocommerce'),
                    'type' => 'text',
                    'description' => __('This information you got in you wellcome email from Swedbank. If you don\'t have this information please fill form and we will contact you or you can contact us by above provided contact information', 'woocommerce'),
                    'desc_tip' => true,
                ),
                'testpass_lv' => array(
                    'title' => __('Test Password for Latvia', 'woocommerce'),
                    'type' => 'text',
                    'description' => __('This information you got in you wellcome email from Swedbank. If you don\'t have this information please fill form and we will contact you or you can contact us by above provided contact information', 'woocommerce'),
                    'desc_tip' => true,
                ),
                'vtid_lv' => array(
                    'title' => __('ID / vTID for Latvia', 'woocommerce'),
                    'type' => 'text',
                    'default' => __('', 'woocommerce'),
                    'desc_tip' => true,
                ),
                'pass_lv' => array(
                    'title' => __('Password for Latvia', 'woocommerce'),
                    'type' => 'text',
                    'default' => __('', 'woocommerce'),
                    'desc_tip' => true,
                ),
                'card_ee' => array(
                    'title' => __('Enable/Disable', 'woocommerce'),
                    'type' => 'checkbox',
                    'label' => __('Card payment for Estonia', 'woocommerce'),
                    'default' => 'no'
                ),
                'swedbank_v2_ee' => array(
                    'title' => __('Enable/Disable', 'woocommerce'),
                    'type' => 'checkbox',
                    'label' => __('Swedbank for Estonia', 'woocommerce'),
                    'default' => 'no'
                ),
                'testmode_ee' => array(
                    'title' => __('Enable/Disable', 'woocommerce'),
                    'type' => 'checkbox',
                    'label' => __('Test mode', 'woocommerce'),
                    'default' => 'no'
                ),
                /*'banklinklng_ee' => array(
                    'title' => __('Banklink language on bank for Estonia', 'woocommerce'),
                    'type' => 'select',
                    'options' => array(
                        'ET' => 'ET',
                        'RU' => 'RU',
                        'EN' => 'EN'
                    ),
                ),*/
                'testvtid_ee' => array(
                    'title' => __('Test ID / vTID for Estonia', 'woocommerce'),
                    'type' => 'text',
                    'description' => __('This information you got in you wellcome email from Swedbank. If you don\'t have this information please fill form and we will contact you or you can contact us by above provided contact information', 'woocommerce'),
                    'desc_tip' => true,
                ),
                'testpass_ee' => array(
                    'title' => __('Test Password for Estonia', 'woocommerce'),
                    'type' => 'text',
                    'description' => __('This information you got in you wellcome email from Swedbank. If you don\'t have this information please fill form and we will contact you or you can contact us by above provided contact information', 'woocommerce'),
                    'desc_tip' => true,
                ),
                'vtid_ee' => array(
                    'title' => __('ID / vTID for Estonia', 'woocommerce'),
                    'type' => 'text',
                    'default' => __('', 'woocommerce'),
                    'desc_tip' => true,
                ),
                'pass_ee' => array(
                    'title' => __('Password for Estonia', 'woocommerce'),
                    'type' => 'text',
                    'default' => __('', 'woocommerce'),
                    'desc_tip' => true,
                )
            );
        }

        /**
                    $this->doDone(true, $order, $list->ID);
         * Output for the order received page.
         *
         * @param int $order_id
         */
        public function thankyou_page($order_id)
        {
            if ($this->instructions) {
                echo wpautop(wptexturize(wp_kses_post($this->instructions)));
            }
            $this->bank_details($order_id);
        }

        public function cronjob()
        {
            include "includes/query.php";

            $query = new swedbank_v2_query($this);

            global $wpdb;

            $date_from = date('Y-m-d H:i:s', strtotime('now -3 hours'));
            $date_to = date('Y-m-d H:i:s', strtotime('now'));
            $post_status = implode("','", array('wc-processing', 'wc-on-hold', 'wc-pending'));

            $result = $wpdb->get_results("SELECT * FROM $wpdb->posts  LEFT JOIN `{$wpdb->prefix}swedbank_orderlist` ON ID = orderidcart
            WHERE post_status IN ('{$post_status}')
            AND post_date BETWEEN '{$date_from}' AND '{$date_to}'  ");

            //$result = $wpdb->get_results("SELECT * FROM  `{$wpdb->prefix}swedbank_orderlist` ");

           // print_r($result);
            if(!empty($result)){
                foreach ($result as $list){

                    $order = wc_get_order($list->ID);

                    $rez = $query->query($list->merchantreference, $order->get_payment_method());
                    //var_dump($rez);
                    if($rez){
                        $this->doDone(true, $order, $list->ID, $rez);
                    }


                }
            }


        }

        /**
         * @return mixed
         */
        public function notification()
        {
            include "includes/query.php";

            $query = new swedbank_v2_query($this);

            $xml = trim(file_get_contents('php://input'));
            try {
                $object = new SimpleXMLElement($xml);
            } catch (Exception $exc) {
                die('<Response>OK</Response>');
            }

            if (isset($object) && isset($object->Event) && isset($object->Event->Purchase)) {

                $oId = $object->Event->Purchase[0]->attributes()['TransactionId'];

                $order = wc_get_order(explode('_', $oId)[1]);

                $rez = $query->query($oId, $order->get_payment_method());

                if($rez){
                    $this->doDone(true, $order, $oId, $rez);
                }
                //-----

            }

            die('<Response>OK</Response>');
        }

        /**
         * Add content to the WC emails.
         *
         * @param WC_Order $order
         * @param bool $sent_to_admin
         * @param bool $plain_text
         */
        public function email_instructions($order, $sent_to_admin, $plain_text = false)
        {

            if (!$sent_to_admin && ('swedbank_v2_card_lt' === $order->get_payment_method() || 'swedbank_v2_card_lv' === $order->get_payment_method() || 'swedbank_v2_card_ee' === $order->get_payment_method())) {
                $html = '';
                if ($_POST['status'] === 'SUCCESS' && !empty($_POST['authcode'])) {
                    $authcode = stripcslashes($_POST['authcode']);
                    $pan = stripcslashes($_POST['pan']);
                    $fulfill_date = stripcslashes($_POST['fulfill_date']);
                    $merchant_reference = stripcslashes($_POST['merchant_reference']);

                        $html = '<b>'.__('Shop URL', 'swedbank-plugin').':</b> ' . home_url() . '<br>
                                <b>'.__('Transaction Date', 'swedbank-plugin').':</b> ' . date("Y-m-d H:i:s", strtotime($fulfill_date)) . '<br>
                                <b>'.__('Order ID', 'swedbank-plugin').': </b>' . $merchant_reference . '<br>
                                <b>'.__('Paid By', 'swedbank-plugin').':</b> ' . $pan . '<br>
                                <b>'.__('Auth Code', 'swedbank-plugin').':</b> ' . $authcode . '<br>';

                } else { }

                if ($this->instructions) {
                    echo wpautop(wptexturize($this->instructions)) . PHP_EOL;
                }

                if (!empty($html)) {
                    echo wpautop(wptexturize($html)) . PHP_EOL;
                }

                $this->bank_details($order->get_id());
            }
        }

        /**
         * Get bank details and place into a list format.
         *
         * @param int $order_id
         */
        private function bank_details($order_id = '')
        {

            if (empty($this->account_details)) {
                return;
            }

            // Get order and store in $order
            $order = wc_get_order($order_id);

            // Get the order country and country $locale
            $country = $order->get_billing_country();
            $locale = $this->get_country_locale();

            // Get sortcode label in the $locale array and use appropriate one
            $sortcode = isset($locale[$country]['sortcode']['label']) ? $locale[$country]['sortcode']['label'] : __('Sort code', 'woocommerce');

            $swedbank_v2_lt_accounts = apply_filters('woocommerce_swedbank_v2_lt_accounts', $this->account_details);

            if (!empty($swedbank_v2_lt_accounts)) {
                $account_html = '';
                $has_details = false;

                foreach ($swedbank_v2_lt_accounts as $swedbank_v2_lt_account) {
                    $swedbank_v2_lt_account = (object)$swedbank_v2_lt_account;

                    if ($swedbank_v2_lt_account->account_name) {
                        $account_html .= '<h3 class="wc-swedbank_v2_lt-bank-details-account-name">' . wp_kses_post(wp_unslash($swedbank_v2_lt_account->account_name)) . ':</h3>' . PHP_EOL;
                    }

                    $account_html .= '<ul class="wc-swedbank_v2_lt-bank-details order_details swedbank_v2_lt_details">' . PHP_EOL;

                    // swedbank_v2_lt account fields shown on the thanks page and in emails
                    $account_fields = apply_filters('woocommerce_swedbank_v2_lt_account_fields', array(
                        'bank_name' => array(
                            'label' => __('Bank', 'woocommerce'),
                            'value' => $swedbank_v2_lt_account->bank_name,
                        ),
                        'account_number' => array(
                            'label' => __('Account number', 'woocommerce'),
                            'value' => $swedbank_v2_lt_account->account_number,
                        ),
                        'sort_code' => array(
                            'label' => $sortcode,
                            'value' => $swedbank_v2_lt_account->sort_code,
                        ),
                        'iban' => array(
                            'label' => __('IBAN', 'woocommerce'),
                            'value' => $swedbank_v2_lt_account->iban,
                        ),
                        'bic' => array(
                            'label' => __('BIC', 'woocommerce'),
                            'value' => $swedbank_v2_lt_account->bic,
                        ),
                    ), $order_id);

                    foreach ($account_fields as $field_key => $field) {
                        if (!empty($field['value'])) {
                            $account_html .= '<li class="' . esc_attr($field_key) . '">' . wp_kses_post($field['label']) . ': <strong>' . wp_kses_post(wptexturize($field['value'])) . '</strong></li>' . PHP_EOL;
                            $has_details = true;
                        }
                    }

                    $account_html .= '</ul>';
                }

                if ($has_details) {
                    echo '<section class="woocommerce-swedbank_v2_lt-bank-details"><h2 class="wc-swedbank_v2_lt-bank-details-heading">' . __('Our bank details', 'woocommerce') . '</h2>' . PHP_EOL . $account_html . '</section>';
                }
            }
        }

        /**
         * Process the payment and return the result.
         *
         * @param int $order_id
         * @return array
         */
        public function process_payment($order_id)
        {
            require_once "includes/class-wc-gateway-swedbank-integration.php";
            global $wpdb;

            $order = wc_get_order($order_id);

            $ob = new WC_Gateway_Swedbank_Integration($order, $this);
            $home_url = home_url();

            $url = $ob->get_url($home_url);

            if (!$url) {
                return array(
                    'result' => 'failure',
                    'message' => "<ul class=\"woocommerce-error\">\n\t\t\t<li><strong>Error:<\/strong> payment method failed.<\/li>\n\t\t\t<\/ul>\n",
                    "refresh" => false,
                    "reload" => false
                );
            } else {


                //$url[1] = $wpdb->_real_escape($url[1]);
                //$url[2] = $wpdb->_real_escape($url[2]);

                //$wpdb->get_results("INSERT INTO `{$wpdb->prefix}swedbank_orderlist` ('orderidcart', 'merchantreference') VALUES ('{$url[1]}', '{$url[2]}') ");
                $wpdb->insert($wpdb->prefix.'swedbank_orderlist',['orderidcart' =>$url[1], 'merchantreference' => $url[2]]);

                return array(
                    'result' => 'success',
                    'redirect' => $url[0],
                );
            }
        }

        public function doDone($n = false, $ob = null, $idn = null, $result = null)
        {
            require_once "includes/class-wc-gateway-swedbank-integration.php";

            if(!$this->settings['order_status'] || empty($this->settings['order_status'])){
                $this->settings['order_status'] = 'wc-completed';
            }

            $order_status_list = wc_get_order_statuses();


            if($ob === null){
                $id = explode('_', $_GET['order_id'])[1];
                $pmmm = $_GET['pmmm'];
                $order = wc_get_order($id);
            } else {
                $id = $idn;
                $pmmm = str_replace('swedbank_v2_','', $ob->get_payment_method());
                $order = $ob;
            }

            if ($order->get_status() == 'wc-completed') {
                if (!$n)
                    wp_redirect($order->get_checkout_order_received_url());
                exit;
            }

            $success = false;

            $ob = new WC_Gateway_Swedbank_Integration($order, $this);


            if ($pmmm === 'swedbank_v2_card_lt' || $pmmm === 'swedbank_v2_card_lv' || $pmmm === 'swedbank_v2_card_ee') {
                if($result === null)
                    $result = $ob->getDone();
                if ($result === 1) {
                    $success = true;
                    $order->update_status($this->settings['order_status'], __($order_status_list[$this->settings['order_status']], 'woocommerce'));
                    // Reduce stock levels
                    wc_reduce_stock_levels($id);
                }
            } else {
                if($result === null)
                    $result = $ob->getDoneB();

                //AUTHORISED
                //REQUIRES_INVESTIGATION
                //PENDING
                //CANCELLED
                //Failed   Cancelled   Pending payment  Completed

                //$order->update_status('on-hold', __('Requires investigation', 'woocommerce'));
                //die;

                //$result = 'PENDING';

                if ($result === 'AUTHORISED' || $result === 'ACCEPTED') {
                    $success = true;
                    $order->update_status($this->settings['order_status'], __($order_status_list[$this->settings['order_status']], 'woocommerce'));
                    wc_reduce_stock_levels($id);
                } else if ($result === 'REQUIRES_INVESTIGATION') {

                    $order->update_status('on-hold', __('Requires investigation', 'woocommerce'));
                    if (!$n) {
                        wc_add_notice(__('Requires investigation:', 'woocommerce') . ' automatic confirmation failed. Need manual confirmation. Please contact shop administrator and provide this order ID: ' . $id . '. ', 'error');
                        if (!$n)
                            wp_redirect($order->get_checkout_payment_url());
                        exit;
                    }
                } else if ($result === 'PENDING') {
                    //pending
                    $success = true;
                    $order->update_status('processing', __('Processing', 'woocommerce'));
                    //wc_reduce_stock_levels($id);
                    WC()->cart->empty_cart();
                    if (!$n) {
                        wc_add_notice(__('Payment in progress:', 'woocommerce') . ' Waiting for confirmation from bank.', 'success');
                        if (!$n)
                            wp_redirect($order->get_checkout_order_received_url());
                        exit;
                    }
                } else if ($result === 'CANCELLED') {
                    $order->update_status('cancelled', __('Cancelled', 'woocommerce'));
                    if (!$n) {
                        wc_add_notice(__('Canceled:', 'woocommerce') . ' payment is canceled.', 'notice');
                        if (!$n)
                            wp_redirect($order->get_cancel_order_url());
                        exit;
                    }
                } else {
                    $order->update_status('failed', __('Failed', 'woocommerce'));
                }
            }


            if ($success) {
                //$order->update_status('on-hold', __('Waiting confirmation', 'woocommerce'));
                // Remove cart
                WC()->cart->empty_cart();
                if (!$n) {
                    wp_redirect($order->get_checkout_order_received_url());
                    exit;
                }
            } else {
                if (!$n) {
                    wc_add_notice(__('Payment error:', 'woocommerce') . ' please try later.', 'error');
                    wp_redirect($order->get_checkout_payment_url());
                    exit;
                }
            }
        }

        public function doFinish()
        {
            $id = explode('_', $_GET['order_id'])[1];
            $order = wc_get_order($id);

            if ($_POST['status'] === 'SUCCESS') {
                $card_data = stripcslashes($_POST['authcode']);

                if (isset($card_data)) {
                    $i = $card_data;
                } else {
                    $i = '';
                }


                if (!empty($i)) {
                    $order->payment_complete($i);
                }
            }
        }

        public function generate_label_html($key, $data)
        {
            $field = $this->plugin_id . $this->id . '_' . $key;
            $defaults = array(
                'class' => 'button-secondary',
                'css' => '',
                'custom_attributes' => array(),
                'desc_tip' => false,
                'description' => '',
                'title' => '',
            );

            $data = wp_parse_args($data, $defaults);

            ob_start();
            ?>


            <tr valign="top">
                <th scope="row" class="titledesc">
                    <label >Please add this URL to CronJob. Set to run this url each 10 minutes.</label>
                </th>
                <td class="forminp">
                    <?php echo home_url(); ?>?swedbankv2=cronjob
                </td>
            </tr>
            <tr>
                <th><label>Notification URL</label></th>
                <td>Please provide this link to Swedbank: <?php echo home_url(); ?>?swedbankv2=notification</td>
            </tr>


            <?php
            return ob_get_clean();
        }

    }

}

add_filter('query_vars', 'swedbank_v2_return');

/**
 *   Add the 'swedbankv2' query variable so WordPress
 *   won't remove it.
 */
function swedbank_v2_return($vars)
{
    $vars[] = "swedbank";
    return $vars;
}

/**
 *   check for  'swedbankv2' query variable and do what you want if its there
 */
add_action('template_redirect', 'swedbank_v2_done');

function swedbank_v2_done($template)
{

    global $wp_query;

    if (!isset($_GET['swedbankv2'])) {
        return $template;
    }


    if ($_GET['swedbankv2'] == 'done') {
        $WC_Gateway_Swedbank_lt_v2 = new WC_Gateway_Swedbank_lt_v2();
        echo '<pre>';
        $WC_Gateway_Swedbank_lt_v2->doDone();

        echo "</pre>";
        exit;
    } else if ($_GET['swedbankv2'] == 'finish') {
        $WC_Gateway_Swedbank_lt_v2 = new WC_Gateway_Swedbank_lt_v2();
        echo '<pre>';
        $WC_Gateway_Swedbank_lt_v2->doFinish();

        echo "</pre>";
        exit;
    } else if ($_GET['swedbankv2'] == 'cronjob') {
        $WC_Gateway_Swedbank_lt_v2 = new WC_Gateway_Swedbank_lt_v2();
        echo '<pre>';
        $WC_Gateway_Swedbank_lt_v2->cronjob();

        echo "</pre>";
        exit;
    }  else if ($_GET['swedbankv2'] == 'notification') {
        $WC_Gateway_Swedbank_lt_v2 = new WC_Gateway_Swedbank_lt_v2();
        echo '<pre>';
        $WC_Gateway_Swedbank_lt_v2->notification();

        echo "</pre>";
        exit;
    }


    return $template;
}

register_activation_hook ( __FILE__, 'on_activate' );

function on_activate() {
    global $wpdb;
    $create_table_query = "
            CREATE TABLE IF NOT EXISTS `{$wpdb->prefix}swedbank_orderlist` (
              `orderidcart` VARCHAR(25),
              `merchantreference` VARCHAR(25) 
            ) ENGINE=MyISAM  DEFAULT CHARSET=utf8;
    ";
    require_once( ABSPATH . 'wp-admin/includes/upgrade.php' );
    dbDelta( $create_table_query );
}